@@ -17,9 +17,11 @@
  * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
  * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+ *
+ * @(#) $Header: /tcpdump/master/tcpslice/machdep.h,v 1.1.1.1 2000/06/11 17:21:45 mcr Exp $ (LBL)
  */
-#ifndef tcpslice_machdep_h
-#define tcpslice_machdep_h
+#ifndef tcpdump_machdep_h
+#define tcpdump_machdep_h
 
-int abort_on_misalignment(char *, const size_t);
+int abort_on_misalignment(char *);
 #endif
