@@ -31,8 +31,8 @@
  * found by ./configure and H.323 session tracking is available only if
  * Libooh323c was found by ./configure. These libraries can be downloaded from:
  *  - http://libnids.sourceforge.net/
- *  - https://www.gnu.org/software/osip/
- *  - https://sourceforge.net/projects/ooh323c/
+ *  - http://www.osip.org/
+ *  - http://ooh323c.sourceforge.net/
  *
  * There are several entry points (from tcpslice.c) to this file:
  *  - sessions_init() has to be called once before any tracking can be done
@@ -43,17 +43,10 @@
  *  - tcp_callback() is called upon reception of correct TCP data
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
-
-#include "varattrs.h"
 #include "sessions.h"
-#include "diag-control.h"
 
 /*
  * The global variables below have the following purposes:
@@ -74,7 +67,7 @@
  *
  * `track_sessions' is a flag set by sessions_init() and
  * sessions_exit() but it is mostly used in tcpslice.c to know
- * whether or not to pass each PCAP frame to libnids in order to
+ * wether or not to pass each PCAP frame to libnids in order to
  * track sessions (it saves further processing if the user did
  * not want to track sessions, which should be the case most of
  * the time).
@@ -106,49 +99,47 @@ uint32_t			sessions_count = 0;
 char				*sessions_file_format = NULL;
 time_t				sessions_expiration_delay = 0;
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #ifndef HAVE_LIBNIDS
 
-void
-sessions_init(const char *types _U_)
+void				sessions_init(char *types)
 {
-  error("libnids required for session tracking support, sorry.");
+  fprintf(stderr, "Libnids required for session tracking support, sorry.\n");
+  exit(-1);
 }
 
 void				sessions_exit(void)
 {
 }
 
-void				sessions_nids_init(pcap_t *p _U_)
+void				sessions_nids_init(pcap_t *p)
 {
 }
 
-#else /* HAVE_LIBNIDS */
+#else
 
 # include <string.h>
 # include <nids.h>
 # ifdef HAVE_LIBOSIPPARSER2
 #  include <osip2/osip.h>
 #  include <osipparser2/sdp_message.h>
-# endif /* HAVE_LIBOSIPPARSER2 */
+# endif
 # ifdef HAVE_LIBOOH323C
 #  include <ooh323.h>
 #  include <ooCalls.h>
 #  include <printHandler.h>
-#  include <ooh323ep.h>
-# endif /* HAVE_LIBOOH323C */
+# endif
 # include "tcpslice.h"
-# include <netinet/ip.h>
-# define IPHDRLEN (sizeof(struct ip))
-# include <netinet/udp.h>
-# define UDPHDRLEN (sizeof(struct udphdr))
-# include <netinet/tcp.h>
-# define TCPHDRLEN (sizeof(struct tcphdr))
-# include <netinet/in.h>
-# include <arpa/inet.h>
+# include "ip.h"
+# include "udp.h"
+# include "tcp.h"
 
 /*
  * Session type identifiers, used as bitmasks for
- * convenience in searches among tracked sessions.
+ * convenience in searchs among tracked sessions.
  */
 enum type
 {
@@ -209,27 +200,25 @@ struct session
   uint64_t			bytes;
   struct session		*next;
   struct session		*prev;
-# if defined(HAVE_LIBOSIPPARSER2) || defined(HAVE_LIBOOH323C)
   union
   {
-#  ifdef HAVE_LIBOSIPPARSER2
+# ifdef HAVE_LIBOSIPPARSER2
     struct
     {
       struct tuple4		rtp_addr;
       osip_call_id_t		*call_id;
       int			picked_up;
     } sip_params;
-#  endif /* HAVE_LIBOSIPPARSER2 */
-#  ifdef HAVE_LIBOOH323C
+# endif
+# ifdef HAVE_LIBOOH323C
     struct
     {
       struct tuple4		cs_addr;
       H225RequestSeqNum		seqnum;
       char			call_id[16];
     } ras_params;
-#  endif /* HAVE_LIBOOH323C */
+# endif
   } u;
-# endif /* defined(HAVE_LIBOSIPPARSER2) || defined(HAVE_LIBOOH323C) */
 };
 
 /*
@@ -286,18 +275,18 @@ static unsigned int		dumper_fd_count = 0;
  * ITU's H.225 Registration Admission Status and H.225 Call Signaling (both
  * part of H.323) data.
  */
-static struct session		*sessions_add(const uint8_t t, const struct tuple4 *addr, const struct session *parent);
+static struct session		*sessions_add(enum type t, struct tuple4 *addr, struct session *parent);
 static void			sessions_del(struct session *elt);
-static struct session		*sessions_find(struct session *start, const uint8_t t, const uint32_t parent_id, const struct tuple4 *addr);
-static struct shared_dumper	*dumper_open(const enum type t, const uint32_t id);
+static struct session		*sessions_find(struct session *start, enum type t, uint32_t parent_id, struct tuple4 *addr);
+static struct shared_dumper	*dumper_open(enum type t, uint32_t id);
 static void			dumper_too_many_open_files(struct shared_dumper **d);
 static void			dumper_close(struct shared_dumper *d);
-static void			dump_frame(const u_char *data, const int len, struct shared_dumper *output);
+static void			dump_frame(u_char *data, int len, struct shared_dumper *output);
 static enum type		parse_type(const char *str);
-static const char		*type2string(const enum type t, const int upper);
-static void			ip_callback(struct ip *ip, int len);
+static const char		*type2string(enum type t, int upper);
+static void			ip_callback(struct rfc_ip *ip, int len);
 static void			tcp_callback(struct tcp_stream *tcp, void **user);
-static void			udp_callback(struct tuple4 *addr, u_char *data, int len, struct ip *ip);
+static void			udp_callback(struct tuple4 *addr, u_char *data, int len, struct rfc_ip *ip);
 static struct session		*sip_callback(struct session *elt, u_char *data, uint32_t len);
 static struct session		*h225_ras_callback(struct session *elt, u_char *data, uint32_t len);
 static struct session		*h225_cs_callback(struct session *elt, u_char *data, uint32_t len);
@@ -309,20 +298,19 @@ static enum type		parse_type(const char *str)
 # ifdef HAVE_LIBOSIPPARSER2
   if (!strcmp("sip", str))
     return CLASS_SIP;
-# endif /* HAVE_LIBOSIPPARSER2 */
+# endif
 # ifdef HAVE_LIBOOH323C
   if (!strcmp("h323", str))
     return CLASS_H323;
-# endif /* HAVE_LIBOOH323C */
-  error("unsupported session type `%s'", str);
-  /* NOTREACHED */
-  return TYPE_NONE;
+# endif
+  fprintf(stderr, "Error: unsupported session type `%s'\n", str);
+  exit(-1);
 }
 
-void
-sessions_init(const char *types)
+void				sessions_init(char *types)
 {
   char				*comma;
+  struct session		*elt;
 
   bonus_time = 0;
   sessions_track_types = TYPE_NONE;
@@ -333,7 +321,7 @@ sessions_init(const char *types)
   }
   sessions_track_types |= parse_type(types);
   while (sessions_count) {
-    struct session *elt = first_session;
+    elt = first_session;
     first_session = first_session->next;
     --sessions_count;
     dumper_close(elt->dumper);
@@ -342,7 +330,7 @@ sessions_init(const char *types)
 # ifdef HAVE_LIBOOH323C
   ooH323EpInitialize(OO_CALLMODE_AUDIOCALL, "/dev/null");
   ooH323EpDisableAutoAnswer();
-# endif /* HAVE_LIBOOH323C */
+# endif
   track_sessions = 1;
 }
 
@@ -378,7 +366,7 @@ void				sessions_exit(void)
 # ifdef HAVE_LIBOSIPPARSER2
     if ((elt->type & TYPE_SIP) && !elt->u.sip_params.picked_up)
       sessions_del(elt);
-# endif /* HAVE_LIBOSIPPARSER2 */
+# endif
   }
 
   /*
@@ -386,10 +374,10 @@ void				sessions_exit(void)
    */
   if (sessions_count) {
     fprintf(stderr,
-	    "%u unclosed %s (id, type, last, source, destination, bytes):\n",
+	    "%d unclosed %s (id, type, last, source, destination, bytes):\n",
 	    sessions_count, sessions_count > 1 ? "sessions" : "session");
     while (NULL != first_session) {
-      fprintf(stderr, "#%u\t", first_session->id);
+      fprintf(stderr, "#%d\t", first_session->id);
       fprintf(stderr, "%s\t", type2string(first_session->type, 1));
       fprintf(stderr, "%s\t", timestamp_to_string(&first_session->lastseen));
       fprintf(stderr, "%15s:%-5d\t",
@@ -398,7 +386,7 @@ void				sessions_exit(void)
       fprintf(stderr, "%15s:%-5d\t",
 	  inet_ntoa(*((struct in_addr *)&first_session->addr.daddr)),
 	  first_session->addr.dest);
-      fprintf(stderr, "%12" PRIu64 "\n", first_session->bytes);
+      fprintf(stderr, "%12d\n", first_session->bytes);
       dumper_close(first_session->dumper);
       if (NULL != first_session->next) {
 	first_session = first_session->next;
@@ -420,22 +408,15 @@ void				sessions_nids_init(pcap_t *p)
   nids_params.pcap_desc = p;
   nids_params.tcp_workarounds = 1;
   if (!nids_init()) {
-    error("%s(): %s", __func__, nids_errbuf);
+    fprintf(stderr, "nids_init: %s\n", nids_errbuf);
+    exit(-1);
   }
-  /*
-   * These conversions between function pointer and void pointer upset GCC,
-   * Clang and XL C.
-   * See also: https://gcc.gnu.org/bugzilla/show_bug.cgi?id=83584
-   */
-  DIAG_OFF_PEDANTIC
   nids_register_ip(ip_callback);
   nids_register_udp(udp_callback);
   nids_register_tcp(tcp_callback);
-  DIAG_ON_PEDANTIC
 }
 
-static struct session *
-sessions_add(const uint8_t t, const struct tuple4 *addr, const struct session *parent)
+static struct session		*sessions_add(enum type t, struct tuple4 *addr, struct session *parent)
 {
   struct session		*elt;
   static uint32_t		counter = 0;
@@ -471,7 +452,7 @@ sessions_add(const uint8_t t, const struct tuple4 *addr, const struct session *p
   first_session = elt;
   ++sessions_count;
   if (verbose && (!elt->parent_id || verbose > 1))
-    printf("Session #%u (%s) opened at %s (active sessions total: %u)\n",
+    printf("Session #%d (%s) opened at %s (active sessions total: %d)\n",
 	elt->parent_id ? elt->parent_id : elt->id,
 	type2string(t, 1),
 	timestamp_to_string(&nids_last_pcap_header->ts),
@@ -487,7 +468,7 @@ static void			sessions_del(struct session *elt)
     return;
   --sessions_count;
   if ((bonus_time || verbose) && (!elt->parent_id || verbose > 1))
-    printf("Session #%u (%s) closed at %s (active sessions total: %u)\n",
+    printf("Session #%d (%s) closed at %s (active sessions total: %d)\n",
 	elt->parent_id ? elt->parent_id : elt->id,
 	type2string(elt->type, 1),
 	timestamp_to_string(&nids_last_pcap_header->ts),
@@ -521,14 +502,13 @@ static void			sessions_del(struct session *elt)
   if (elt->type & TYPE_SIP)
     if (NULL != elt->u.sip_params.call_id)
       osip_call_id_free(elt->u.sip_params.call_id);
-# endif /* HAVE_LIBOSIPPARSER2 */
+# endif
 
   dumper_close(elt->dumper);
   free(elt);
 }
 
-static struct session *
-sessions_find(struct session *start, const uint8_t t, const uint32_t parent_id, const struct tuple4 *addr)
+static struct session		*sessions_find(struct session *start, enum type t, uint32_t parent_id, struct tuple4 *addr)
 {
   struct session		*elt;
 
@@ -555,8 +535,7 @@ sessions_find(struct session *start, const uint8_t t, const uint32_t parent_id,
   return NULL;
 }
 
-static struct shared_dumper *
-dumper_open(const enum type t, const uint32_t id)
+static struct shared_dumper	*dumper_open(enum type t, uint32_t id)
 {
   struct shared_dumper		*d;
 
@@ -590,9 +569,11 @@ static void			dumper_too_many_open_files(struct shared_dumper **d)
     (*d)->filedesc = pcap_dump_open(nids_params.pcap_desc, (*d)->filename);
   }
   if (NULL == (*d)->filedesc) {
-    error("%s(): %s: %s", __func__,
+    fprintf(stderr,
+	"pcap_dump_open: %s: %s\n",
 	(*d)->filename,
 	pcap_geterr(nids_params.pcap_desc));
+    exit(-1);
   }
 }
 
@@ -610,8 +591,7 @@ static void			dumper_close(struct shared_dumper *d)
   }
 }
 
-static const char *
-type2string(const enum type t, const int upper)
+static const char		*type2string(enum type t, int upper)
 {
   if (t & TYPE_SIP)
     return upper ? "SIP" : "sip";
@@ -624,8 +604,7 @@ type2string(const enum type t, const int upper)
   return "???";
 }
 
-static void
-dump_frame(const u_char *data, const int len, struct shared_dumper *output)
+static void			dump_frame(u_char *data, int len, struct shared_dumper *output)
 {
   u_char			*frame;
   struct pcap_pkthdr		ph;
@@ -662,31 +641,28 @@ dump_frame(const u_char *data, const int len, struct shared_dumper *output)
  * mobilize resources, we give TCP sessions 60 seconds to complete
  * the TCP handshake or else they are considered to be closed.
  */
-static void			ip_callback(struct ip *ip, int len)
+static void			ip_callback(struct rfc_ip *ip, int len)
 {
   struct tuple4			addr;
-  struct tcphdr			*tcp;
+  struct rfc_tcp		*tcp;
   struct session		*elt;
   struct session		*elt_next;
   unsigned int			ip_data_offset = IPHDRLEN;
 
-  if (len < 0)
-    error("%s(): len < 0", __func__);
-
   for (elt = first_session; NULL != elt; elt = elt_next) {
     elt_next = elt->next;
     if (elt->timeout && (nids_last_pcap_header->ts.tv_sec >= elt->timeout))
       sessions_del(elt);
   }
-  if ((ip->ip_hl > 5) && ((ip->ip_hl * 4) <= len))
-    ip_data_offset = ip->ip_hl * 4;
-  if ((ip->ip_p != 6) || ((unsigned)len < (ip_data_offset + TCPHDRLEN)))
+  if ((ip->ihl > 5) && ((ip->ihl * 4) <= len))
+    ip_data_offset = ip->ihl * 4;
+  if ((ip->protocol != 6) || (len < (ip_data_offset + TCPHDRLEN)))
     return; /* not TCP or too short */
-  tcp = (struct tcphdr *)((char *)ip + ip_data_offset);
-  addr.saddr = *((u_int *)&ip->ip_src);
-  addr.daddr = *((u_int *)&ip->ip_dst);
-  addr.source = ntohs(tcp->th_sport);
-  addr.dest = ntohs(tcp->th_dport);
+  tcp = (struct rfc_tcp *)((char *)ip + ip_data_offset);
+  addr.saddr = *((u_int *)&ip->saddr);
+  addr.daddr = *((u_int *)&ip->daddr);
+  addr.source = ntohs(tcp->sport);
+  addr.dest = ntohs(tcp->dport);
   if (NULL != (elt = sessions_find(first_session, TYPE_TCP, 0, &addr))) {
     dump_frame((u_char *)ip, len, elt->dumper);
     if (sessions_expiration_delay)
@@ -694,7 +670,7 @@ static void			ip_callback(struct ip *ip, int len)
     elt->lastseen = nids_last_pcap_header->ts;
     return;
   }
-  if (!(tcp->th_flags & TH_SYN) || bonus_time)
+  if (!tcp->syn || bonus_time)
     return;
   if (addr.source == 5060 || addr.dest == 5060)
     elt = sessions_add(TYPE_TCP | TYPE_SIP, &addr, NULL);
@@ -707,7 +683,7 @@ static void			ip_callback(struct ip *ip, int len)
   /* 60 seconds to complete TCP handshake */
 }
 
-static void			udp_callback(struct tuple4 *addr, u_char *udp_data, int udp_data_len, struct ip *ip)
+static void			udp_callback(struct tuple4 *addr, u_char *udp_data, int udp_data_len, struct rfc_ip *ip)
 {
   struct session		*elt;
   unsigned int			udp_data_offset = IPHDRLEN + UDPHDRLEN;
@@ -733,8 +709,8 @@ static void			udp_callback(struct tuple4 *addr, u_char *udp_data, int udp_data_l
    * a new session object (`elt') might be created by the callback,
    * with a pointer to a different PCAP file.
    */
-  if ((ip->ip_hl > 5) && ((ip->ip_hl * 4U) < (udp_data_offset + udp_data_len)))
-    udp_data_offset = ip->ip_hl * 4 + UDPHDRLEN;
+  if ((ip->ihl > 5) && ((ip->ihl * 4) < (udp_data_offset + udp_data_len)))
+    udp_data_offset = ip->ihl * 4 + UDPHDRLEN;
   dump_frame((u_char *)ip, udp_data_offset + udp_data_len, elt->dumper);
 }
 
@@ -782,17 +758,12 @@ static void			tcp_callback(struct tcp_stream *tcp, void **user)
   }
 }
 
-# ifndef HAVE_LIBOSIPPARSER2
-static struct session			*sip_callback(struct session *sip, u_char *data _U_, uint32_t len _U_)
-{
-  return sip;
-}
-# else /* HAVE_LIBOSIPPARSER2 */
-static int
-sip_get_address(const osip_message_t *msg, u_int *host, u_short *port)
+# ifdef HAVE_LIBOSIPPARSER2
+static int		sip_get_address(osip_message_t *msg, u_int *host, u_short *port)
 {
   osip_content_type_t	*ctt;
   sdp_message_t		*sdp;
+  char			*tmp;
   int			i;
   int			j;
 
@@ -804,10 +775,10 @@ sip_get_address(const osip_message_t *msg, u_int *host, u_short *port)
     return 0;
   if (osip_strcasecmp(ctt->subtype, "sdp"))
     return 0;
-  for (i = 0; !osip_list_eol(&msg->bodies, i); ++i) {
+  for (i = 0; !osip_list_eol(msg->bodies, i); ++i) {
     sdp = NULL;
     sdp_message_init(&sdp);
-    char *tmp = ((osip_body_t *)osip_list_get(&msg->bodies, i))->body;
+    tmp = ((osip_body_t *)osip_list_get(msg->bodies, i))->body;
     if (sdp_message_parse(sdp, tmp)) {
       sdp_message_free(sdp);
       continue;
@@ -831,6 +802,7 @@ sip_get_address(const osip_message_t *msg, u_int *host, u_short *port)
 static struct session			*sip_callback(struct session *sip, u_char *data, uint32_t len)
 {
   osip_message_t			*msg;
+  struct tuple4				addr;
   struct session			*start;
   struct session			*rtp;
   osip_call_id_t			*call_id;
@@ -924,19 +896,14 @@ static struct session			*sip_callback(struct session *sip, u_char *data, uint32_
   osip_message_free(msg);
   return sip;
 }
-# endif /* HAVE_LIBOSIPPARSER2 */
-
-# ifndef HAVE_LIBOOH323C
-static struct session			*h225_ras_callback(struct session *ras, u_char *data _U_, uint32_t len _U_)
+# else
+static struct session			*sip_callback(struct session *sip, u_char *data, uint32_t len)
 {
-  return ras;
+  return sip;
 }
+# endif
 
-static struct session			*h225_cs_callback(struct session *cs, u_char *data _U_, uint32_t len _U_)
-{
-  return cs;
-}
-# else /* HAVE_LIBOOH323C */
+# ifdef HAVE_LIBOOH323C
 static struct session			*h225_ras_callback(struct session *ras, u_char *data, uint32_t len)
 {
   OOCTXT				ctxt;
@@ -1012,13 +979,8 @@ static struct session			*h225_ras_callback(struct session *ras, u_char *data, ui
   return ras;
 }
 
-static struct session			*h225_cs_callback(struct session *cs, u_char *data, uint32_t len _U_)
+static struct session			*h225_cs_callback(struct session *cs, u_char *data, uint32_t len)
 {
-  /* ooCreateCall() for some reason declares its read-only first argument as
-   * "char *" instead of "const char *", wrap the string in an array in order
-   * not to upset the compiler.
-   */
-  char					callType[] = "incoming";
   char					callToken[20];
   OOH323CallData			*call;
   Q931Message				q931;
@@ -1028,7 +990,7 @@ static struct session			*h225_cs_callback(struct session *cs, u_char *data, uint
   struct tuple4				addr;
 
   ooGenerateCallToken(callToken, 20);
-  call = ooCreateCall(callType, callToken);
+  call = ooCreateCall("incoming", callToken);
   call->pH225Channel = (OOH323Channel*) memAllocZ (call->pctxt, sizeof (OOH323Channel));
   if (OO_OK == ooQ931Decode(call, &q931, ntohs(*((u_short *)(data + 2))) - 4, data + 4)) {
     if (OO_OK == ooHandleH2250Message(call, &q931)) {
@@ -1064,6 +1026,17 @@ static struct session			*h225_cs_callback(struct session *cs, u_char *data, uint
   ooCleanCall(call);
   return cs;
 }
-# endif /* HAVE_LIBOOH323C */
 
-#endif /* HAVE_LIBNIDS */
+# else
+static struct session			*h225_ras_callback(struct session *ras, u_char *data, uint32_t len)
+{
+  return ras;
+}
+
+static struct session			*h225_cs_callback(struct session *cs, u_char *data, uint32_t len)
+{
+  return cs;
+}
+# endif
+
+#endif
